from __future__ import absolute_import

from django.core import validators
from django.core.exceptions import ValidationError
from django.forms import ChoiceField
from django.forms.models import ModelChoiceField, ModelMultipleChoiceField

from .widgets import ChainedSelect, ChainedSelectMultiple


class ChainedChoiceField(ChoiceField):
    def __init__(self, parent_field, ajax_url, choices=None, empty_label='--------', *args, **kwargs):
        self.parent_field = parent_field
        self.ajax_url = ajax_url
        self.choices = choices or (('', empty_label), )
        self.empty_label = empty_label

        defaults = {
            'widget': ChainedSelect(parent_field=parent_field, ajax_url=ajax_url, attrs={'empty_label': empty_label}),
        }
        defaults.update(kwargs)

        super(ChainedChoiceField, self).__init__(choices=self.choices, *args, **defaults)

    def valid_value(self, value):
        """Dynamic choices so just return True for now"""
        return True


class ChainedModelChoiceField(ModelChoiceField):
    def __init__(self, parent_field, ajax_url, model, empty_label='--------', *args, **kwargs):
        self.parent_field = parent_field
        self.ajax_url = ajax_url
        self.model = model
        # self.queryset = model.objects.all()  # Large querysets could take long time to load all values (django-cities)
        self.queryset = model.objects.none()
        self.empty_label = empty_label

        defaults = {
            'widget': ChainedSelect(parent_field=parent_field, ajax_url=ajax_url, attrs={'empty_label': empty_label}),
        }
        defaults.update(kwargs)

        super(ChainedModelChoiceField, self).__init__(queryset=self.queryset, empty_label=empty_label, *args, **defaults)

    def valid_value(self, value):
        """Dynamic choices so just return True for now"""
        return True

    def to_python(self, value):
        empty_values = getattr(self, 'empty_values', list(validators.EMPTY_VALUES))
        if value in empty_values:
            return None
        try:
            key = self.to_field_name or 'pk'
            value = self.model.objects.get(**{key: value})
        except (ValueError, self.queryset.model.DoesNotExist):
            raise ValidationError(self.error_messages['invalid_choice'], code='invalid_choice')
        return value

    def validate(self, value):
        """
        Validates that the input is in self.choices.
        """
        super(ChoiceField, self).validate(value)
        if value and not self.valid_value(value):
            raise ValidationError(
                self.error_messages['invalid_choice'],
                code='invalid_choice',
                params={'value': value},
            )


class ChainedModelMultipleChoiceField(ModelMultipleChoiceField):
    def __init__(self, parent_field, ajax_url, model, *args, **kwargs):
        self.parent_field = parent_field
        self.ajax_url = ajax_url
        self.model = model
        self.queryset = model.objects.all()

        defaults = {
            'widget': ChainedSelectMultiple(parent_field=parent_field, ajax_url=ajax_url),
        }
        defaults.update(kwargs)

        super(ChainedModelMultipleChoiceField, self).__init__(queryset=self.queryset, *args, **defaults)



from django.forms.widgets import Select, SelectMultiple


class ChainedSelectMixin(object):
    """
    A ChoiceField widget mixin where the options for the select are dependent on the value of the parent select field.
    When the parent field is changed, an ajax call is made to determine the options.

    Form must inherit from ChainedChoicesMixin (or from helper forms ChainedChoicesForm and ChainedChoicesModelForm)
    which loads the options when there is already an instance or initial data.
    """

    template_name = 'clever_selects/widgets/chained_select.html'

    def __init__(self, parent_field=None, ajax_url=None, *args, **kwargs):
        self.parent_field = parent_field
        self.ajax_url = ajax_url
        super(ChainedSelectMixin, self).__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        context = super(ChainedSelectMixin, self).get_context(name, value, attrs)

        field_prefix = attrs['id'][:attrs['id'].rfind('-') + 1]

        if not field_prefix:
            parent_field_id = "id_" + self.parent_field
        else:
            parent_field_id = field_prefix + self.parent_field

        context['widget']['attrs']['ajax_url'] = self.ajax_url
        context['js_parent_field_id'] = parent_field_id
        context['js_chained_id'] = attrs['id']
        return context


class ChainedSelect(ChainedSelectMixin, Select):
    pass


class ChainedSelectMultiple(ChainedSelectMixin, SelectMultiple):
    pass



import logging

from django import template
from django.middleware.csrf import get_token
from django.conf import settings
from django.core.files.storage import get_storage_class
from django.utils.safestring import mark_safe

staticfiles_storage = get_storage_class(settings.STATICFILES_STORAGE)()

register = template.Library()

log = logging.getLogger('clever_selects')


@register.simple_tag(takes_context=True)
def clever_selects_js_import(context, csrf=True):
    """ Return the js script tag for the clever-selects.js file.
    If the csrf argument is present and it's ``nocsrf`` clever-selects will not
    try to mark the request as if it need the csrf token. By default use
    the clever_selects_js_import template tag will make django set the csrftoken
    cookie on the current request."""

    csrf = csrf != 'nocsrf'
    request = context.get('request')

    if request and csrf:
        get_token(request)
    elif csrf:
        log.warning("The 'request' object must be accessible within the context. "
                    "You must add 'django.contrib.messages.context_processors.request' "
                    "to your TEMPLATE_CONTEXT_PROCESSORS and render your views using a RequestContext.")

    url = staticfiles_storage.url('js/clever-selects.js')
    return mark_safe('<script src="%s" type="text/javascript" charset="utf-8"></script>' % url)



'''
@author: Erik Telepovsky
'''

import json

from django.core.exceptions import ObjectDoesNotExist
from django.core.validators import EMPTY_VALUES
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.utils.cache import add_never_cache_headers
from django.views.generic.base import View


class ChainedSelectFormViewMixin(object):

    def get_form_kwargs(self):
        kwargs = super(ChainedSelectFormViewMixin, self).get_form_kwargs()
        kwargs.update({'user': self.request.user})
        return kwargs


class ChainedSelectChoicesView(View):
    child_set = None

    def dispatch(self, request, *args, **kwargs):
        self.field = request.GET.get("field")
        self.field_value = request.GET.get("field_value", None)
        self.parent_field = request.GET.get("parent_field")
        self.parent_value = request.GET.get("parent_value")
        if self.parent_value in EMPTY_VALUES + ('None', ):
            return self.empty_response()
        return super(ChainedSelectChoicesView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        response = HttpResponse(
            json.dumps(self.get_choices(), cls=DjangoJSONEncoder),
            content_type='application/javascript'
        )
        add_never_cache_headers(response)
        return response

    def empty_response(self):
        response = HttpResponse(
            json.dumps((), cls=DjangoJSONEncoder),
            content_type='application/javascript'
        )
        add_never_cache_headers(response)
        return response

    def get_child_set(self):
        return self.child_set

    def get_choices(self):
        choices = []
        if self.parent_value in EMPTY_VALUES + ('None', ) or self.get_child_set() is None:
            return []
        try:
            for obj in self.get_child_set().all():
                choices.append((obj.pk, str(obj)))
            return choices
        except ObjectDoesNotExist:
            return []



from __future__ import absolute_import

import json

from django import forms
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import ObjectDoesNotExist

try:
    from django.urls import reverse, resolve
except:
    from django.core.urlresolvers import reverse, resolve

from django.core.validators import EMPTY_VALUES
from django.db import models
from django.http.request import HttpRequest
from django.utils.encoding import smart_str, force_text

from .form_fields import ChainedChoiceField, ChainedModelChoiceField, ChainedModelMultipleChoiceField


class ChainedChoicesMixin(object):
    """
    Form Mixin to be used with ChainedChoicesForm and ChainedChoicesModelForm.
    It loads the options when there is already an instance or initial data.
    """
    user = None
    prefix = None
    fields = []
    chained_fields_names = []
    chained_model_fields_names = []

    def init_chained_choices(self, *args, **kwargs):
        self.chained_fields_names = self.get_fields_names_by_type(ChainedChoiceField)
        self.chained_model_fields_names = self.get_fields_names_by_type(ChainedModelChoiceField) + self.get_fields_names_by_type(ChainedModelMultipleChoiceField)
        self.user = kwargs.get('user', self.user)

        if kwargs.get('data', None) is not None:
            self.set_choices_via_ajax(kwargs['data'])

        elif len(args) > 0 and args[0] not in EMPTY_VALUES:
            self.set_choices_via_ajax(args[0])

        elif kwargs.get('instance', None) is not None:
            oldest_parent_field_names = list(set(self.get_oldest_parent_field_names()))
            youngest_child_names = list(set(self.get_youngest_children_field_names()))

            for youngest_child_name in youngest_child_names:
                self.find_instance_attr(kwargs['instance'], youngest_child_name)

            for oldest_parent_field_name in oldest_parent_field_names:
                try:
                    self.fields[oldest_parent_field_name].initial = getattr(self, '%s' % oldest_parent_field_name)
                except AttributeError:
                    pass

            self.set_choices_via_ajax()

        elif 'initial' in kwargs and kwargs['initial'] not in EMPTY_VALUES:
            self.set_choices_via_ajax(kwargs['initial'], is_initial=True)
        else:
            for field_name in self.chained_fields_names + self.chained_model_fields_names:
                empty_label = self.fields[field_name].empty_label
                self.fields[field_name].choices = [('', empty_label)]

    def set_choices_via_ajax(self, kwargs=None, is_initial=False):
        for field_name in self.chained_fields_names + self.chained_model_fields_names:
            field = self.fields[field_name]

            try:
                if kwargs is not None:
                    # initial data do not have any prefix
                    if self.prefix in EMPTY_VALUES or is_initial:
                        parent_value = kwargs.get(field.parent_field, None)
                        field_value = kwargs.get(field_name, None)
                    else:
                        parent_value = kwargs.get('%s-%s' % (self.prefix, field.parent_field), None)
                        field_value = kwargs.get('%s-%s' % (self.prefix, field_name), None)
                else:
                    parent_value = self.initial.get(field.parent_field, None)
                    field_value = self.initial.get(field_name, None)

                    if parent_value is None:
                        parent_value = getattr(self, '%s' % field.parent_field, None)

                    if field_value is None:
                        field_value = getattr(self, '%s' % field_name, None)

                field.choices = [('', field.empty_label)]

                # check that parent_value is valid
                if parent_value:
                    parent_value = getattr(parent_value, 'pk', parent_value)

                    url = force_text(field.ajax_url)
                    params = {
                        'field_name': field_name,
                        'parent_value': parent_value,
                        'field_value': field_value
                    }

                    # This will get the callable from the url.
                    # All we need to do is pass in a 'request'
                    url_callable = resolve(url).func

                    # Build the fake request
                    fake_request = HttpRequest()
                    fake_request.META["SERVER_NAME"] = "localhost"
                    fake_request.META["SERVER_PORT"] = '80'

                    # Add parameters and user if supplied
                    fake_request.method = "GET"
                    for key, value in params.items():
                        fake_request.GET[key] = value

                    if hasattr(self, "user") and self.user:
                        fake_request.user = self.user
                    else:
                        fake_request.user = AnonymousUser()

                    # Get the response
                    response = url_callable(fake_request)

                    # Apply the data (if it's returned)
                    if smart_str(response.content):
                        try:
                            field.choices += json.loads(smart_str(response.content))
                        except ValueError:
                            raise ValueError('Data returned from request (url={url}, params={params}) could not be deserialized to Python object'.format(
                                url=url,
                                params=params
                            ))

                field.initial = field_value

            except ObjectDoesNotExist:
                field.choices = ()

    def get_fields_names_by_type(self, type_):
        result = []
        for field_name in self.fields:
            field = self.fields[field_name]
            if isinstance(field, type_):
                result.append(field_name)
        return result

    def get_parent_fields_names(self):
        result = []
        for field_name in self.fields:
            field = self.fields[field_name]
            if hasattr(field, 'parent_field'):
                result.append(field.parent_field)
        return result

    def get_children_field_names(self, parent_name):
        if parent_name in EMPTY_VALUES:
            return []
        result = []
        for field_name in self.fields:
            field = self.fields[field_name]
            if getattr(field, 'parent_field', None) == parent_name:
                result.append(field_name)
        return result

    def get_chained_fields_names(self):
        chained_fields_names = self.get_fields_names_by_type(ChainedChoiceField)
        chained_model_fields_names = self.get_fields_names_by_type(ChainedModelChoiceField)
        return chained_fields_names + chained_model_fields_names

    def get_oldest_parent_field_names(self):
        chained_fields_names = self.get_fields_names_by_type(ChainedChoiceField)
        chained_model_fields_names = self.get_fields_names_by_type(ChainedModelChoiceField)

        oldest_parent_field_names = []
        for field_name in self.get_parent_fields_names():
            if field_name not in chained_fields_names and field_name not in chained_model_fields_names:
                oldest_parent_field_names.append(field_name)
        return oldest_parent_field_names

    def get_youngest_children_field_names(self):
        result = []
        chained_fields_names = self.get_fields_names_by_type(ChainedChoiceField)
        chained_model_fields_names = self.get_fields_names_by_type(ChainedModelChoiceField)

        for field_name in chained_fields_names + chained_model_fields_names:
            if field_name not in self.get_parent_fields_names():
                result.append(field_name)
        return result

    def find_instance_attr(self, instance, attr_name):
        field = self.fields[attr_name]
        if hasattr(instance, attr_name):
            attribute = getattr(instance, attr_name)
            attr_value = getattr(attribute, 'pk', smart_str(attribute)) if attribute else None
            setattr(self, '%s' % attr_name, attr_value)

            if hasattr(field, 'parent_field'):
                parent_instance = attribute if isinstance(attribute, models.Model) else instance
                self.find_instance_attr(parent_instance, field.parent_field)


class ChainedChoicesForm(forms.Form, ChainedChoicesMixin):
    """
    Form class to be used with ChainedChoiceField and ChainedSelect widget
    If there is request POST data in *args (i.e. form validation was invalid)
    then the options will be loaded when the form is built.
    """

    def __init__(self, language_code=None, *args, **kwargs):
        if kwargs.get('user'):
            self.user = kwargs.pop('user')  # To get request.user. Do not use kwargs.pop('user', None) due to potential security hole
        super(ChainedChoicesForm, self).__init__(*args, **kwargs)
        self.language_code = language_code
        self.init_chained_choices(*args, **kwargs)

    def is_valid(self):
        if self.language_code:
            # response is not translated to requested language code :/
            # so translation is triggered manually
            from django.utils.translation import activate
            activate(self.language_code)
        return super(ChainedChoicesForm, self).is_valid()


class ChainedChoicesModelForm(forms.ModelForm, ChainedChoicesMixin):
    """
    Form class to be used with ChainedChoiceField and ChainedSelect widget
    If there is already an instance (i.e. editing)
    then the options will be loaded when the form is built.
    """

    def __init__(self, *args, **kwargs):
        if kwargs.get('user'):
            self.user = kwargs.pop('user')  # To get request.user. Do not use kwargs.pop('user', None) due to potential security hole
        super(ChainedChoicesModelForm, self).__init__(*args, **kwargs)
        self.language_code = kwargs.get('language_code', None)
        self.init_chained_choices(*args, **kwargs)

    def is_valid(self):
        if self.language_code:
            # response is not translated to requested language code :/
            # so translation is triggered manually
            from django.utils.translation import activate
            activate(self.language_code)
        return super(ChainedChoicesModelForm, self).is_valid()



#!/usr/bin/env python
from setuptools import setup


setup(
    name='django-clever-selects-xadmin',
    version='0.8.7',
    description='Chained select box widget for Django framework using AJAX requests.',
    long_description=open('README.rst').read(),
    author='Pragmatic Mates',
    author_email='info@pragmaticmates.com',
    maintainer='Croxlink',
    maintainer_email='eddiep@croxlink.com',
    url='https://github.com/CROXLINK/django-clever-selects',
    packages=[
        'clever_selects',
        'clever_selects.templatetags'
    ],
    include_package_data=True,
    install_requires=('django',),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Development Status :: 3 - Alpha'
    ],
    license='BSD License',
    keywords="django clever chained selects",
)




class ChainedSelectAdminMixin(object):

    def get_form(self, request, obj=None, **kwargs):
        form = super(ChainedSelectAdminMixin, self).get_form(request, obj, **kwargs)
        if request.user:
            form.user = request.user
        return form
